<?xml version="1.0"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.1">

<xsl:template match="cluster-info">
<html>
	<head>
		<title><xsl:text>Cluster-Info</xsl:text></title>
		<link rel="stylesheet" href="styles.css" type="text/css"/>
		<link rel="shortcut icon" href="images/favicon.ico"/>
	</head>
	<body>
		<xsl:variable name="num-of-nodes" select="count(//client)"/>
		<xsl:variable name="num-of-cpus" select="count(//cpu)"/>
		<xsl:variable name="sum-mhz" select="sum(//mhz)"/>
		<xsl:variable name="sum-mem" select="sum(//memsize)"/>
		<xsl:variable name="cpus-per-node"
			select="$num-of-cpus div $num-of-nodes"/>
		<xsl:variable name="mhz-per-node" select="$sum-mhz div $num-of-nodes"/>
		<xsl:variable name="mhz-per-cpu" select="$sum-mhz div $num-of-cpus"/>
		<xsl:variable name="mem-per-node" select="$sum-mem div $num-of-nodes"/>
		<xsl:variable name="mem-per-cpu" select="$sum-mem div $num-of-cpus"/>

		<p>
			<h1>BraLUG Cluster Information</h1>
			<b>
				<xsl:text>Summary for </xsl:text>
				<xsl:value-of select="$num-of-nodes"/>
				<xsl:text> nodes:</xsl:text>
			</b><br/><br/>

			<table>
				<!-- header -->
				<tr>
					<th><xsl:text> </xsl:text></th>
					<th>whole cluster</th>
					<th>average per node</th>
					<th>average per cpu</th>
				</tr>

				<tr>
					<th>CPUs</th>
					<td><xsl:value-of select="$num-of-cpus"/></td>
					<td><xsl:value-of
						select="format-number($cpus-per-node, '#.00')"/></td>
					<td><xsl:text>1 ;)</xsl:text></td>	
				</tr>

				<!-- cpu speed -->
				<tr>
					<th>CPU frequency in MHz</th>
					<td><xsl:value-of
						select="format-number($sum-mhz, '#.00')"/></td>
					<td><xsl:value-of
						select="format-number($mhz-per-node, '#.00')"/></td>
					<td><xsl:value-of
						select="format-number($mhz-per-cpu, '#.00')"/></td>
				</tr>

				<!-- ram size -->
				<tr>
					<th>RAM size in MiB</th>
					<td><xsl:value-of
						select="format-number($sum-mem div 1024, '#.00')"
						/></td>
					<td><xsl:value-of
						select="format-number($mem-per-node div 1024, '#.00')"
						/></td>
					<td><xsl:value-of
						select="format-number($mem-per-cpu div 1024, '#.00')"
						/></td>
				</tr>
			</table>
			<br/>

			<!-- flags -->
			<xsl:variable name="percent-amd"
				select="count(//cpu[contains(vendor_id,'AuthenticAMD')]) div $num-of-cpus * 100"/>
			<xsl:variable name="percent-intel"
				select="count(//cpu[contains(vendor_id,'GenuineIntel')]) div $num-of-cpus * 100"/>
			<xsl:variable name="percent-centaur"
				select="count(//cpu[contains(vendor_id,'CentaurHauls')]) div $num-of-cpus * 100"/>
			<xsl:variable name="percent-mmx"
				select="count(//cpu/flags/flag_mmx) div $num-of-cpus * 100"/>
			<xsl:variable name="percent-3dnow"
				select="count(//cpu/flags/flag_3dnow) div $num-of-cpus * 100"/>
			<xsl:variable name="percent-sse"
				select="count(//cpu/flags/flag_sse) div $num-of-cpus * 100"/>

			<table>
				<tr>
					<th> </th>
					<th>percentage of CPUs</th>
				</tr>

				<tr>
					<th></th>
					<th><span class="level2">CPU type and vendor</span></th>
				</tr>
				<tr>
					<th>Intel</th>
					<td><xsl:value-of
						select="format-number($percent-intel,'#.00')"/></td>
				</tr>

				<tr>
					<th>AMD</th>
					<td><xsl:value-of
						select="format-number($percent-amd,'#.00')"/></td>
				</tr>

				<tr>
					<th>Cyrix / VIA</th>
					<td><xsl:value-of
						select="format-number($percent-centaur,'#.00')"/></td>
				</tr>
		
				<!-- extensions -->
				<tr>
					<th></th>
					<th><span class="level2">extensions</span></th>
				</tr>
				<tr>
					<th>MMX (MultiMedia eXtension)</th>
					<td><xsl:value-of
						select="format-number($percent-mmx,'#.00')"/></td>
				</tr>
				
				<tr>
					<th>SSE (Streaming SIMD Extensions)</th>
					<td><xsl:value-of
						select="format-number($percent-sse,'#.00')"/></td>
				</tr>

				<tr>
					<th>3Dnow!</th>
					<td><xsl:value-of
						select="format-number($percent-3dnow,'#.00')"/></td>
				</tr>
			</table>
		</p>
		<hr/>
		<xsl:apply-templates/>
	</body>
</html>
</xsl:template>

<xsl:template match="client">
<p>
	<b>
		<xsl:value-of select="./hostname"/>
		<xsl:text> (MAC: </xsl:text>
		<xsl:value-of select="./network/hwaddr"/>
		<xsl:text>)</xsl:text>
	</b>
	<br/><br/>
	<xsl:apply-templates select="cpus"/>
	<br/>
	<xsl:value-of select="format-number(./memsize div 1024, '#.00')"/>
	<xsl:text> MiB RAM</xsl:text>
	
	<xsl:apply-templates select="benchmarks"/>
</p>
</xsl:template>

<xsl:template match="cpus">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="cpu">
	<xsl:text>CPU #</xsl:text><xsl:value-of select="./num"/>
	<xsl:text>: </xsl:text>
	<xsl:value-of select="./vendor_id"/><xsl:text> </xsl:text>
	<xsl:value-of select="./model_name"/><xsl:text>, </xsl:text>
	<xsl:value-of select="./mhz"/><xsl:text> MHz (</xsl:text>
	<xsl:value-of select="./bogomips"/><xsl:text> BogoMIPS)</xsl:text>
	<xsl:if test="./cache_size!='(unknown)'">
		<xsl:text>, </xsl:text>
		<xsl:value-of select="./cache_size"/>
		<xsl:text> Cache</xsl:text>
	</xsl:if>
	<br/>
</xsl:template>

<xsl:template match="benchmarks">
	<br/><br/>
	<i><xsl:text>benchmarking results:</xsl:text></i><br/>
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="result">
	<xsl:value-of select="./test"/>
	<xsl:text>: </xsl:text>
	<xsl:value-of select="./seconds"/>
	<xsl:text> seconds, </xsl:text>
	<xsl:value-of select="./useconds"/>
	<xsl:text> microseconds</xsl:text>
</xsl:template>

</xsl:stylesheet>
