#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>

#include <pvm3.h>

#include "main.h"
#include "master.h"
#include "client.h"
#include "slist.h"
#include "toxml.h"

char *prog = "cluster-info";

int main(int argc, char *argv[])
{
	int mytid; /* my task id */
	int parentid; /* parent id */
	int nhost;
	SList *clients;

	/* get my task id and check for error */
	mytid = pvm_mytid();
	if(mytid < 0)
	{
		pvm_perror(argv[0]);
		return EXIT_FAILURE;
	}

	printf("%s: common: my TID is %d\n", prog, mytid);

	/* get my parent id and check for error */
	parentid = pvm_parent();
	if((parentid < 0) &&
		(parentid != PvmNoParent) &&		/* run from shell */
		(parentid != PvmParentNotSet))		/* spawned from pvm console */
	{
		printf("%s: master: parentid %d\n", prog, parentid);
		pvm_perror(argv[0]);
		pvm_exit();
		return EXIT_FAILURE;
	}

	/* am i the parent? */
	if((parentid == PvmNoParent) || (parentid == PvmParentNotSet))
	{
		toxml_init(XMLDIR "/cluster-info.xml",
			"cluster-info.xsl");

		nhost = master_spawn(argv[0], &clients);
		master_collect(nhost, clients);

		toxml_close();
	}
	else
	{
		/* i'm a spawned process */

		/* get system information */
		client_information();

		if(pvm_tidtohost(mytid) != pvm_tidtohost(parentid))
		{
			/* don't do benchmarking on master host */

			/* do benchmarking */
			client_benchmark();
		}

		sleep(2);
	}

	return EXIT_SUCCESS;
}
