#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include <unistd.h>

#include <pvm3.h>

#define __USE_MISC
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <net/if.h>

#include "main.h"
#include "misc.h"
#include "network.h"

#define DEBUG 0

int network_info(void)
{
	NetworkInterfaceInfo *ifinfo;

	ifinfo = network_gethwaddr("eth0");
	if(ifinfo)
	{
		pvm_initsend(PvmDataDefault);
		pvm_pkstr(ifinfo->hwaddr);
		pvm_pkbyte(ifinfo->ip, 4, 1);
		pvm_send(pvm_parent(), MSGTAG_ETH0INFO);
	}

	network_freeinfo(ifinfo);

	return EXIT_SUCCESS;
}

NetworkInterfaceInfo *network_gethwaddr(const char *ifname)
{
	char tmpaddr[18];
	int sockfd, i, num_ifs = 0;
	struct ifconf ifc;
	struct if_nameindex *if_names;
	NetworkInterfaceInfo *ifinfo;

	sockfd = socket(PF_INET, SOCK_DGRAM, 0);

	if(sockfd < 0)
	{
		fprintf(stderr, "E: could not create socket...\n");
		return NULL;
	}

	/* get number of interfaces */
	if_names = if_nameindex();
	while(if_names[num_ifs].if_name)
	{
#if DEBUG >= 2
		printf("I: %d: %s\n",
			if_names[num_ifs].if_index,
			if_names[num_ifs].if_name);
#endif
		num_ifs ++;
	}
	if_freenameindex(if_names);

	ifc.ifc_len = sizeof(struct ifreq) * num_ifs;
	ifc.ifc_buf = calloc(sizeof(struct ifreq), num_ifs);

	if(ioctl(sockfd, SIOCGIFCONF, &ifc) < 0)
	{
		fprintf(stderr, "E: SIOCGIFCONF: %s (%d)\n", strerror(errno), errno);
		free(ifc.ifc_buf);
		return NULL;
	}

	for(i = 0; i < num_ifs; i ++)
	{
		if(ioctl(sockfd, SIOCGIFHWADDR, &(ifc.ifc_req[i])) < 0)
		{
			fprintf(stderr, "E: SIOCGIFHWADDR: %s (%d)\n",
				strerror(errno), errno);
			continue;
		}

		memset(tmpaddr, 0, 18 * sizeof(char));
		sprintf(tmpaddr, "%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
			ifc.ifc_req[i].ifr_hwaddr.sa_data[0] & 0xFF,
			ifc.ifc_req[i].ifr_hwaddr.sa_data[1] & 0xFF,
			ifc.ifc_req[i].ifr_hwaddr.sa_data[2] & 0xFF,
			ifc.ifc_req[i].ifr_hwaddr.sa_data[3] & 0xFF,
			ifc.ifc_req[i].ifr_hwaddr.sa_data[4] & 0xFF,
			ifc.ifc_req[i].ifr_hwaddr.sa_data[5] & 0xFF);

		if(strcmp(ifc.ifc_req[i].ifr_name, ifname) == 0)
		{
			ifinfo = m_new0(NetworkInterfaceInfo, 1);
			ifinfo->hwaddr = m_strdup(tmpaddr);
		}

#if DEBUG > 0
		printf("%-5s: hwaddr: %s\n", ifc.ifc_req[i].ifr_name, tmpaddr);
#endif
	}

	free(ifc.ifc_buf);
	close(sockfd);

	return ifinfo;
}

void network_freeinfo(NetworkInterfaceInfo *ifinfo)
{
	assert(ifinfo != NULL);

	if(ifinfo->hwaddr) free(ifinfo->hwaddr);

	free(ifinfo);
}
